@@ -583,6 +583,20 @@ void bindPyTypes(py::module & m)
                DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_ACES_GAMUTMAP_07))
         .value("FIXED_FUNCTION_ACES_GAMUT_COMP_13", FIXED_FUNCTION_ACES_GAMUT_COMP_13,
                DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_ACES_GAMUT_COMP_13))
+        .value("FIXED_FUNCTION_LIN_TO_PQ", FIXED_FUNCTION_LIN_TO_PQ,
+               DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_LIN_TO_PQ))
+        .value("FIXED_FUNCTION_LIN_TO_GAMMA_LOG", FIXED_FUNCTION_LIN_TO_GAMMA_LOG,
+               DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_LIN_TO_GAMMA_LOG))
+        .value("FIXED_FUNCTION_LIN_TO_DOUBLE_LOG", FIXED_FUNCTION_LIN_TO_DOUBLE_LOG,
+               DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_LIN_TO_DOUBLE_LOG))
+        .value("FIXED_FUNCTION_ACES_OUTPUT_TRANSFORM_20", FIXED_FUNCTION_ACES_OUTPUT_TRANSFORM_20,
+               DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_ACES_OUTPUT_TRANSFORM_20))
+        .value("FIXED_FUNCTION_ACES_RGB_TO_JMH_20", FIXED_FUNCTION_ACES_RGB_TO_JMH_20,
+               DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_ACES_RGB_TO_JMH_20))
+        .value("FIXED_FUNCTION_ACES_TONESCALE_COMPRESS_20", FIXED_FUNCTION_ACES_TONESCALE_COMPRESS_20,
+               DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_ACES_TONESCALE_COMPRESS_20))
+        .value("FIXED_FUNCTION_ACES_GAMUT_COMPRESS_20", FIXED_FUNCTION_ACES_GAMUT_COMPRESS_20,
+               DOC(PyOpenColorIO, FixedFunctionStyle, FIXED_FUNCTION_ACES_GAMUT_COMPRESS_20))
         .export_values();
 
     py::enum_<ExposureContrastStyle>(
